@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gconf-internals.h"
 #include "gconf-backend.h"
 #include "gconf-schema.h"
@@ -36,9 +36,12 @@
 #include <time.h>
 #include <math.h>
 
+#include <dbus/dbus.h>
+
 #ifdef G_OS_WIN32
 #include <windows.h>
 #include <share.h>
+#define pipe(fds) _pipe(fds, 4096, _O_BINARY)
 #endif
 
 gboolean gconf_log_debug_messages = FALSE;
@@ -118,7 +121,8 @@ gconf_key_key        (const gchar* key)
   
   end = strrchr(key, '/');
 
-  ++end;
+  if (end != NULL)
+    ++end;
 
   return end;
 }
@@ -159,7 +163,7 @@ gconf_value_from_corba_value(const ConfigValue* value)
       type = GCONF_VALUE_PAIR;
       break;
     default:
-      gconf_log(GCL_DEBUG, "Invalid type in %s", G_GNUC_FUNCTION);
+      gconf_log(GCL_DEBUG, "Invalid type in %s", G_STRFUNC);
       return NULL;
     }
 
@@ -215,7 +219,7 @@ gconf_value_from_corba_value(const ConfigValue* value)
           case BInvalidVal:
             break;
           default:
-            g_warning("Bizarre list type in %s", G_GNUC_FUNCTION);
+            g_warning("Bizarre list type in %s", G_STRFUNC);
             break;
           }
 
@@ -236,7 +240,7 @@ gconf_value_from_corba_value(const ConfigValue* value)
                 if (val == NULL)
                   gconf_log(GCL_ERR, _("Couldn't interpret CORBA value for list element"));
                 else if (val->type != gconf_value_get_list_type(gval))
-                  gconf_log(GCL_ERR, _("Incorrect type for list element in %s"), G_GNUC_FUNCTION);
+                  gconf_log(GCL_ERR, _("Incorrect type for list element in %s"), G_STRFUNC);
                 else
                   list = g_slist_prepend(list, val);
                 
@@ -346,7 +350,7 @@ gconf_fill_corba_value_from_gconf_value(const GConfValue *value,
             
           default:
             cv->_u.list_value.list_type = BInvalidVal;
-            gconf_log(GCL_DEBUG, "Invalid list type in %s", G_GNUC_FUNCTION);
+            gconf_log(GCL_DEBUG, "Invalid list type in %s", G_STRFUNC);
             break;
           }
         
@@ -385,7 +389,7 @@ gconf_fill_corba_value_from_gconf_value(const GConfValue *value,
       break;
     default:
       cv->_d = InvalidVal;
-      gconf_log(GCL_DEBUG, "Unknown type in %s", G_GNUC_FUNCTION);
+      gconf_log(GCL_DEBUG, "Unknown type in %s", G_STRFUNC);
       break;
     }
 }
@@ -657,7 +661,7 @@ gconf_value_type_from_string(const gchar* type_str)
 }
 
 /*
- * Config files (yikes! we can't store our config in GConf!)
+ * Configuration files (yikes! we can't store our configuration in GConf!)
  */
 
 static gchar*
@@ -1095,105 +1099,46 @@ gconf_current_locale(void)
  * Log
  */
 
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-
 void
 gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
 {
-  gchar* msg;
   va_list args;
-#ifdef HAVE_SYSLOG_H
-  gchar* convmsg;
-  int syslog_pri = LOG_DEBUG;
-#endif
+  GLogLevelFlags loglevel;
 
   if (!gconf_log_debug_messages && 
       pri == GCL_DEBUG)
     return;
-  
+
+  switch (pri)
+    {
+    case GCL_EMERG:
+    case GCL_ALERT:
+    case GCL_CRIT:
+      loglevel = G_LOG_LEVEL_ERROR;
+      break;
+    case GCL_ERR:
+      loglevel = G_LOG_LEVEL_CRITICAL;
+      break;
+    case GCL_WARNING:
+      loglevel = G_LOG_LEVEL_WARNING;
+      break;
+    case GCL_NOTICE:
+      loglevel = G_LOG_LEVEL_MESSAGE;
+      break;
+    case GCL_INFO:
+      loglevel = G_LOG_LEVEL_INFO;
+      break;
+    case GCL_DEBUG:
+      loglevel = G_LOG_LEVEL_DEBUG;
+      break;
+    default:
+      g_assert_not_reached();
+      break;
+    }
+
   va_start (args, fmt);
-  msg = g_strdup_vprintf(fmt, args);
+  g_logv (G_LOG_DOMAIN, loglevel, fmt, args);
   va_end (args);
-
-#ifdef HAVE_SYSLOG_H
-  if (gconf_daemon_mode)
-    {
-      switch (pri)
-        {
-        case GCL_EMERG:
-          syslog_pri = LOG_EMERG;
-          break;
-      
-        case GCL_ALERT:
-          syslog_pri = LOG_ALERT;
-          break;
-      
-        case GCL_CRIT:
-          syslog_pri = LOG_CRIT;
-          break;
-      
-        case GCL_ERR:
-          syslog_pri = LOG_ERR;
-          break;
-      
-        case GCL_WARNING:
-          syslog_pri = LOG_WARNING;
-          break;
-      
-        case GCL_NOTICE:
-          syslog_pri = LOG_NOTICE;
-          break;
-      
-        case GCL_INFO:
-          syslog_pri = LOG_INFO;
-          break;
-      
-        case GCL_DEBUG:
-          syslog_pri = LOG_DEBUG;
-          break;
-
-        default:
-          g_assert_not_reached();
-          break;
-        }
-
-      convmsg = g_locale_from_utf8 (msg, -1, NULL, NULL, NULL);
-      if (!convmsg)
-        syslog (syslog_pri, "%s", msg);
-      else
-        {
-	  syslog (syslog_pri, "%s", convmsg);
-	  g_free (convmsg);
-	}
-    }
-  else
-#endif
-    {
-      switch (pri)
-        {
-        case GCL_EMERG:
-        case GCL_ALERT:
-        case GCL_CRIT:
-        case GCL_ERR:
-        case GCL_WARNING:
-          g_printerr ("%s\n", msg);
-          break;
-      
-        case GCL_NOTICE:
-        case GCL_INFO:
-        case GCL_DEBUG:
-          g_print ("%s\n", msg);
-          break;
-
-        default:
-          g_assert_not_reached();
-          break;
-        }
-    }
-  
-  g_free(msg);
 }
 
 /*
@@ -1879,7 +1824,7 @@ gconf_value_decode (const gchar* encoded)
         
         d = g_strtod(s, &endptr);
         if (endptr == s)
-          g_warning("Failure converting string to double in %s", G_GNUC_FUNCTION);
+          g_warning("Failure converting string to double in %s", G_STRFUNC);
         gconf_value_set_float(val, d);
       }
       break;
@@ -2475,107 +2420,117 @@ open_empty_locked_file (const gchar *directory,
   return fd;
 }
 
-static ConfigServer
-read_current_server_and_set_warning (const gchar *iorfile,
-                                     GString     *warning)
+static char *
+get_ior (gboolean start_if_not_found,
+         GString  *failure_log)
 {
-  FILE *fp;
-  
-  fp = g_fopen (iorfile, "r");
-          
-  if (fp == NULL)
-    {
-      if (warning)
-        g_string_append_printf (warning,
-                                _("IOR file '%s' not opened successfully, no gconfd located: %s"),
-                                iorfile, g_strerror (errno));
+        DBusMessage *message, *reply;
+        DBusConnection *connection;
+        DBusError bus_error;
+        char *ior;
 
-      return CORBA_OBJECT_NIL;
-    }
-  else /* successfully opened IOR file */
-    {
-      char buf[2048] = { '\0' };
-      const char *str = NULL;
-      
-      fgets (buf, sizeof (buf) - 2, fp);
-      fclose (fp);
-
-      /* The lockfile format is <pid>:<ior> for gconfd
-       * or <pid>:none for gconftool
-       */
-      str = buf;
-      while (isdigit ((unsigned char) *str))
-        ++str;
-
-      if (*str == ':')
-        ++str;
-          
-      if (str[0] == 'n' &&
-          str[1] == 'o' &&
-          str[2] == 'n' &&
-          str[3] == 'e')
-        {
-          if (warning)
-            g_string_append_printf (warning,
-                                    _("gconftool or other non-gconfd process has the lock file '%s'"),
-                                    iorfile); 
-        }
-      else /* file contains daemon IOR */
-        {
-          CORBA_ORB orb;
-          CORBA_Environment ev;
-          ConfigServer server;
-          
-          CORBA_exception_init (&ev);
-                  
-          orb = gconf_orb_get ();
-
-          if (orb == NULL)
-            {
-              if (warning)
-                g_string_append_printf (warning,
-                                        _("couldn't contact ORB to resolve existing gconfd object reference"));
-              return CORBA_OBJECT_NIL;
-            }
-                  
-          server = CORBA_ORB_string_to_object (orb, (char*) str, &ev);
-          CORBA_exception_free (&ev);
-
-          if (server == CORBA_OBJECT_NIL &&
-              warning)
-            g_string_append_printf (warning,
-                                    _("Failed to convert IOR '%s' to an object reference"),
-                                    str);
-          
-          return server;
+        /* if the bus isn't running and we don't want to start gconfd then
+         * we don't want to autolaunch the bus either, so bail early.
+         */
+        if (g_getenv ("DBUS_SESSION_BUS_ADDRESS") == NULL &&
+           (!start_if_not_found || g_getenv ("DISPLAY") == NULL)) {
+                if (failure_log)
+                    g_string_append_printf (failure_log,
+                                            _("Not running within active session"));
+                return NULL;
         }
 
-      return CORBA_OBJECT_NIL;
-    }
+        dbus_error_init (&bus_error);
+        connection = dbus_bus_get (DBUS_BUS_SESSION, &bus_error);
+
+        if (dbus_error_is_set (&bus_error)) {
+                if (failure_log)
+                    g_string_append_printf (failure_log,
+                                            _("Failed to get connection to session: %s"),
+                                            bus_error.message);
+                dbus_error_free (&bus_error);
+                return NULL;
+        }
+
+        message = dbus_message_new_method_call ("org.gnome.GConf",
+                                                "/org/gnome/GConf",
+                                                "org.gnome.GConf",
+                                                "GetIOR");
+        dbus_message_set_auto_start (message, start_if_not_found);
+
+        reply = dbus_connection_send_with_reply_and_block (connection, message, -1,
+                                                           &bus_error);
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&bus_error)) {
+                if (failure_log)
+                    g_string_append_printf (failure_log,
+                                            _("Could not send message to gconf daemon: %s"),
+                                            bus_error.message);
+                dbus_error_free (&bus_error);
+                return NULL;
+        }
+
+        ior = NULL;
+        if (!dbus_message_get_args (reply, &bus_error, DBUS_TYPE_STRING,
+                                    &ior, DBUS_TYPE_INVALID)) {
+                if (failure_log)
+                    g_string_append_printf (failure_log,
+                                            _("daemon gave erroneous reply: %s"),
+                                            bus_error.message);
+                dbus_error_free (&bus_error);
+                return NULL;
+        }
+
+        ior = g_strdup (ior);
+
+        dbus_message_unref (reply);
+        dbus_connection_unref (connection);
+
+        return ior;
 }
 
 static ConfigServer
-read_current_server (const gchar *iorfile,
-                     gboolean     warn_if_fail)
+gconf_get_server (gboolean  start_if_not_found,
+                  GString  *failure_log)
 {
-  GString *warning;
   ConfigServer server;
-  
-  if (warn_if_fail)
-    warning = g_string_new (NULL);
-  else
-    warning = NULL;
+  char *ior;
+  CORBA_ORB orb;
+  CORBA_Environment ev;
 
-  server = read_current_server_and_set_warning (iorfile, warning);
+  ior = get_ior (start_if_not_found, failure_log);
 
-  if (warning)
+  if (ior == NULL)
     {
-      if (warning->len > 0)
-	gconf_log (GCL_WARNING, "%s", warning->str);
-
-      g_string_free (warning, TRUE);
+      return CORBA_OBJECT_NIL;
     }
 
+  CORBA_exception_init (&ev);
+  orb = gconf_orb_get ();
+
+  if (orb == NULL)
+    {
+      if (failure_log)
+        {
+          g_string_append_printf (failure_log,
+                                _("couldn't contact ORB to resolve existing gconfd object reference"));
+          g_free (ior);
+        }
+      return CORBA_OBJECT_NIL;
+    }
+
+  server = CORBA_ORB_string_to_object (orb, (char*) ior, &ev);
+  CORBA_exception_free (&ev);
+
+  if (server == CORBA_OBJECT_NIL &&
+      failure_log)
+    g_string_append_printf (failure_log,
+                            _("Failed to convert IOR '%s' to an object reference"),
+                            ior);
+
+  g_free (ior);
+
   return server;
 }
 
@@ -2620,7 +2575,7 @@ gconf_get_lock_or_current_holder (const gchar  *lock_directory,
        * it to the caller. Error is already set.
        */
       if (current_server)
-        *current_server = read_current_server (lock->iorfile, TRUE);
+        *current_server = gconf_get_server (FALSE, NULL);
 
       gconf_lock_destroy (lock);
       
@@ -2783,40 +2738,6 @@ gconf_release_lock (GConfLock *lock,
   return retval;
 }
 
-/* This function doesn't try to see if the lock is valid or anything
- * of the sort; it just reads it. It does do the object_to_string
- */
-ConfigServer
-gconf_get_current_lock_holder  (const gchar *lock_directory,
-                                GString     *failure_log)
-{
-  char *iorfile;
-  ConfigServer server;
-
-  iorfile = g_strconcat (lock_directory, "/ior", NULL);
-  server = read_current_server_and_set_warning (iorfile, failure_log);
-  g_free (iorfile);
-  return server;
-}
-
-void
-gconf_daemon_blow_away_locks (void)
-{
-  char *lock_directory;
-  char *iorfile;
-  
-  lock_directory = gconf_get_lock_dir ();
-
-  iorfile = g_strconcat (lock_directory, "/ior", NULL);
-
-  if (g_unlink (iorfile) < 0)
-    g_printerr (_("Failed to unlink lock file %s: %s\n"),
-                iorfile, g_strerror (errno));
-
-  g_free (iorfile);
-  g_free (lock_directory);
-}
-
 static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
 
 CORBA_ORB
@@ -2875,10 +2796,17 @@ gconf_get_daemon_dir (void)
     {
       char *s;
       char *subdir;
+      const char *tmpdir;
 
       subdir = g_strconcat ("gconfd-", g_get_user_name (), NULL);
       
-      s = g_build_filename (g_get_tmp_dir (), subdir, NULL);
+      if (g_getenv ("GCONF_TMPDIR")) {
+	tmpdir = g_getenv ("GCONF_TMPDIR");
+      } else {
+	tmpdir = g_get_tmp_dir ();
+      }
+      
+      s = g_build_filename (tmpdir, subdir, NULL);
 
       g_free (subdir);
 
@@ -2908,174 +2836,44 @@ gconf_get_lock_dir (void)
   return lock_dir;
 }
 
-#if defined (F_SETFD) && defined (FD_CLOEXEC)
-
-#ifndef HAVE_FDWALK
-static void
-set_cloexec (gint fd)
-{
-  fcntl (fd, F_SETFD, FD_CLOEXEC);
-#else
-static int
-set_cloexec (void *data, int fd)
-{
-  int *pipes = (int *)data;
-
-  if (fd != pipes[1] && fd > 2)
-    fcntl (fd, F_SETFD, FD_CLOEXEC);
-
-  return 0;
-#endif
-}
-
-
-static void
-close_fd_func (gpointer data)
-{
-  int *pipes = data;
-  
-#ifndef HAVE_FDWALK
-  gint open_max;
-  gint i;
-  
-  open_max = sysconf (_SC_OPEN_MAX);
-  for (i = 3; i < open_max; i++)
-    {
-      /* don't close our write pipe */
-      if (i != pipes[1])
-        set_cloexec (i);
-    }
-#else
-  (void) fdwalk(set_cloexec, (void *)pipes);
-#endif
-}
-
-#else
-
-#define close_fd_func NULL
-
-#endif
-
 ConfigServer
 gconf_activate_server (gboolean  start_if_not_found,
                        GError  **error)
 {
   ConfigServer server = CORBA_OBJECT_NIL;
-  int p[2] = { -1, -1 };
-  char buf[1];
-  GError *tmp_err;
-  char *argv[3];
-  char *gconfd_dir;
-  char *lock_dir;
   GString *failure_log;
-  struct stat statbuf;
   CORBA_Environment ev;
-  gboolean dir_accessible;
 
   failure_log = g_string_new (NULL);
   
-  gconfd_dir = gconf_get_daemon_dir ();
+  g_string_append (failure_log, " 1: ");
+  server = gconf_get_server (start_if_not_found, failure_log);
+
+  /* Confirm server exists */
+  CORBA_exception_init (&ev);
+
+  if (!CORBA_Object_is_nil (server, &ev))
+    {
+      ConfigServer_ping (server, &ev);
   
-  dir_accessible = g_stat (gconfd_dir, &statbuf) >= 0;
-
-  if (!dir_accessible && errno != ENOENT)
-    {
-      server = CORBA_OBJECT_NIL;
-      gconf_log (GCL_WARNING, _("Failed to stat %s: %s"),
-		 gconfd_dir, g_strerror (errno));
-    }
-  else if (dir_accessible)
-    {
-      g_string_append (failure_log, " 1: ");
-      lock_dir = gconf_get_lock_dir ();
-      server = gconf_get_current_lock_holder (lock_dir, failure_log);
-      g_free (lock_dir);
-
-      /* Confirm server exists */
-      CORBA_exception_init (&ev);
-
-      if (!CORBA_Object_is_nil (server, &ev))
-	{
-	  ConfigServer_ping (server, &ev);
-      
-	  if (ev._major != CORBA_NO_EXCEPTION)
-	    {
-	      server = CORBA_OBJECT_NIL;
-
-	      g_string_append_printf (failure_log,
-				      _("Server ping error: %s"),
-				      CORBA_exception_id (&ev));
-	    }
-	}
-
-      CORBA_exception_free (&ev);
-  
-      if (server != CORBA_OBJECT_NIL)
-	{
-	  g_string_free (failure_log, TRUE);
-	  g_free (gconfd_dir);
-	  return server;
-	}
-    }
-
-  g_free (gconfd_dir);
-
-  if (start_if_not_found)
-    {
-      /* Spawn server */
-      if (pipe (p) < 0)
+      if (ev._major != CORBA_NO_EXCEPTION)
         {
-          g_set_error (error,
-                       GCONF_ERROR,
-                       GCONF_ERROR_NO_SERVER,
-                       _("Failed to create pipe for communicating with spawned gconf daemon: %s\n"),
-                       g_strerror (errno));
-          goto out;
+          server = CORBA_OBJECT_NIL;
+
+          g_string_append_printf (failure_log,
+                                  _("Server ping error: %s"),
+                                  CORBA_exception_id (&ev));
         }
-
-      argv[0] = g_build_filename (GCONF_SERVERDIR, GCONFD, NULL);
-      argv[1] = g_strdup_printf ("%d", p[1]);
-      argv[2] = NULL;
-  
-      tmp_err = NULL;
-      if (!g_spawn_async (NULL,
-                          argv,
-                          NULL,
-                          G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
-                          close_fd_func,
-			  p,
-                          NULL,
-                          &tmp_err))
-        {
-          g_free (argv[0]);
-          g_free (argv[1]);
-          g_set_error (error,
-                       GCONF_ERROR,
-                       GCONF_ERROR_NO_SERVER,
-                       _("Failed to launch configuration server: %s\n"),
-                       tmp_err->message);
-          g_error_free (tmp_err);
-          goto out;
-        }
-      
-      g_free (argv[0]);
-      g_free (argv[1]);
-
-      /* If the server dies, we don't want to block indefinitely in
-	 the read. */
-      close (p[1]);
-      p[1] = -1;
-
-      /* Block until server starts up */
-      read (p[0], buf, 1);
-
-      g_string_append (failure_log, " 2: ");
-      lock_dir = gconf_get_lock_dir ();
-      server = gconf_get_current_lock_holder (lock_dir, failure_log);
-      g_free (lock_dir);
     }
-  
- out:
+
+  CORBA_exception_free (&ev);
+
+  if (server != CORBA_OBJECT_NIL)
+    {
+      g_string_free (failure_log, TRUE);
+      return server;
+    }
+
   if (server == CORBA_OBJECT_NIL &&
       error &&
       *error == NULL)
@@ -3087,11 +2885,6 @@ gconf_activate_server (gboolean  start_if_not_found,
 
   g_string_free (failure_log, TRUE);
   
-  if (p[0] != -1)
-    close (p[0]);
-  if (p[1] != -1)
-    close (p[1]);
-  
   return server;
 }
 
